@@ -5,8 +5,8 @@ allocate_child_to_potential <- function(potC, x, cliques, child, parents) {
     family_in_Ck <- all(c(child, parents) %in% cliques[[k]])
     if (family_in_Ck) {
       if (is.null(potC$C[[k]])) {
-        unity <- sparta::sparta_unity_struct(attr(x, "dim_names")[cliques[[k]]])
-        potC$C[[k]] <- sparta::mult(cpt, unity)
+        # unity <- sparta::sparta_unity_struct(attr(x, "dim_names")[cliques[[k]]])
+        potC$C[[k]] <- cpt # sparta::mult(cpt, unity)
       } else {
         potC$C[[k]] <- sparta::mult(potC$C[[k]], cpt)
       }
@@ -16,17 +16,29 @@ allocate_child_to_potential <- function(potC, x, cliques, child, parents) {
   NULL
 }
 
+## broadcast_clique_potentials <- function(potC, x, cliques) {
+##   for (k in seq_along(cliques)) {
+##     pot_k <- potC[["C"]][[k]]
+##     dim_k <- sparta::dim_names(pot_k)
+##     Ck    <- cliques[[k]]
+##     full  <- setequal(names(dim_k), Ck)
+##     if (!full) {
+##       unity <- sparta::sparta_unity_struct(attr(x, "dim_names")[Ck])      
+##       potC$C[[k]] <- sparta::mult(pot_k, unity)
+##     }
+##   }
+## }
+
 new_charge <- function(x, cliques, parents) {
   # x: cpt_list
   potC <- new.env()
   potC[["C"]] <- vector("list", length(cliques))
 
   children <- names(parents)
-
   for (child in children) {
     allocate_child_to_potential(potC, x, cliques, child, parents[[child]])
   }
-  
+
   # Some clique potentials may be empty due to triangulation
   # We set these as the identity = 1 for all configurations
   is_null <- .map_lgl(potC[["C"]], is.null)
@@ -39,6 +51,8 @@ new_charge <- function(x, cliques, parents) {
     }
   }
 
+  # broadcast_clique_potentials(potC, x, cliques)
+  
   names_potS <- paste("S", 1:length(cliques), sep = "")
   potS <- structure(vector("list", length(cliques)), names = names_potS)
   names(potC$C) <- names(cliques)
